/* code03e8.c - EvtProcessSoftKeyStroke handler
   Copyright C 2000 Richard Cohen
   This file is a part of Dial Hack
Previous copyright:
   Copyright  199 Ben Darnell
   This file is a part of LinkMaster,
   which is distributed under the GNU GPL.
   Based on SampleHack by Roger Chaplin <foursquaredev@att.net>
*/

#include <Common.h>
#include <SysAll.h>
#include <UIAll.h>
#include <FeatureMgr.h>
/* gcc doesnt like that header file, so here's the decl i need: */
#include <UI/ScrDriverNew.h>
#include <System/AudioLib.h>
#include "dial.h"
#include "dialRsc.h"
Err EvtFlushNextPenStroke() SYS_TRAP(sysTrapEvtFlushNextPenStroke);


void DoClipDial();
void DoSelectDial();

// This is the code that will be called whenever ANY app calls EvtGetEvent,
// and myhack is enabled in HackMaster. HackMaster is not smart about the
// content of this file; it will simply jump to the first executable statement
// in this file. If you need helper functions, prototype them above, and
// define them AFTER the trap patch.
Err EvtProcessSoftKeyStroke(PointType* start, PointType* end)
{
   DWord               temp;
   Err              (*oldTrap)(PointType*, PointType*);

   PenBtnInfoPtr btninfo;
   UInt numbuttons=0, i;
   UInt startbutton=0, endbutton=0;
  
   // get the address of the old EvtGetEvent() from HackMaster
   FtrGet(MYCRID, 0x3e8, &temp);
   // set the function pointer to the old trap
   oldTrap = (Err (*)(PointType*, PointType*)) temp;

   btninfo=EvtGetPenBtnList(&numbuttons);
   for (i=0; i<numbuttons; i++) {
      if (RctPtInRectangle(start->x, start->y, &btninfo[i].boundsR))
	 startbutton=btninfo[i].asciiCode;
      if (RctPtInRectangle(end->x, end->y, &btninfo[i].boundsR))
	 endbutton=btninfo[i].asciiCode;
   }

   if (startbutton!=endbutton) {
      MyPrefsType prefs;
      Word prefssize=sizeof(prefs);

      if ((PrefGetAppPreferences('Dhck', 0, &prefs, &prefssize, true)
	   ==noPreferenceFound)||(prefssize<sizeof(prefs))) {
	 prefs.selectstart=calcChr;
	 prefs.selectend=findChr;
     prefs.clipstart=findChr;
     prefs.clipend=calcChr;
     prefs.flags = 0x0000;
      }
      if ((startbutton==prefs.selectstart)&&(endbutton==prefs.selectend)) {
	    DoSelectDial();
	    return 0;
      }
      if ((startbutton==prefs.clipstart)&&(endbutton==prefs.clipend)) {
	    DoClipDial();
	    return 0;
      }
   }

   return oldTrap(start, end);
}

void DoSelectDial()
{
	Err err;
	Word libRef;
	VoidHand lochand;
	CharPtr locstr, fldstr;
	FormPtr form;
	Word focuspoint, startpoint, endpoint;
	FieldPtr fieldp;
	FormObjectKind objkind;
	Boolean loaded=false;


	// Click to confirm to the user
//	SndPlaySystemSound(sndClick);

	err = SysLibFind(AudioLibName, &libRef);

	if (err)
	{
		err = SysLibLoad(AudioLibTypeID, AudioLibCreatorID, &libRef);
		if (err)
		{
			return;
		}
		else
		{
			loaded=true;
		}
	}

	AudioLibOpen(libRef);

	form=FrmGetActiveForm();
	if (form==NULL)
	{
		goto cleanup;
	}
	focuspoint=FrmGetFocus(form);
	if (focuspoint==noFocus)
	{
		goto cleanup;
	}
	objkind=FrmGetObjectType(form, focuspoint);
	if (objkind==frmFieldObj)
	{
		fieldp=(FieldPtr)FrmGetObjectPtr(form, focuspoint);
	}
	else if (objkind==frmTableObj)
	{
		fieldp=TblGetCurrentField (FrmGetObjectPtr (form, focuspoint));
	}
	else
	{
		goto cleanup;
	}

	FldGetSelection(fieldp, &startpoint, &endpoint);
	if (!(startpoint<endpoint))
	{
		goto cleanup;
	}

	lochand=MemHandleNew(endpoint-startpoint+1);
	locstr=MemHandleLock(lochand);
	fldstr=FldGetTextPtr(fieldp);
	StrNCopy(locstr, fldstr+startpoint, endpoint-startpoint);
	locstr[endpoint-startpoint]='\0';

	AudioPlayDTMFStr(libRef, locstr, 2, 3);
	MemHandleUnlock(lochand);
	MemHandleFree(lochand);
	

cleanup:
	AudioLibClose(libRef);
	if (loaded) SysLibRemove(libRef);

}
void DoClipDial()
{
	Err err;
	Word libRef;
	Word cliplength;
	VoidHand cliphand, lochand;
	CharPtr locstr, clipstr;


	// Click to confirm to the user
	SndPlaySystemSound(sndClick);

	err = SysLibFind(AudioLibName, &libRef);

	if (err)
	{
		err = SysLibLoad(AudioLibTypeID, AudioLibCreatorID, &libRef);
		if (err)
		{
			return;
		}
	}

	AudioLibOpen(libRef);

	cliphand=ClipboardGetItem(clipboardText, &cliplength);
	if (cliplength==0) goto cleanup;

	lochand=MemHandleNew(cliplength+1);
	locstr=MemHandleLock(lochand);
	clipstr=MemHandleLock(cliphand);
	StrNCopy(locstr, clipstr, cliplength);
	MemHandleUnlock(cliphand);
	AudioPlayDTMFStr(libRef, locstr, 2, 3);
	MemHandleUnlock(lochand);
	MemHandleFree(lochand);

cleanup:
	AudioLibClose(libRef);
	SysLibRemove(libRef);

}
