/* code07d0.c - Preferences panel
   Copyright C 2000 Richard Cohen
   This file is a part of Dial Hack
   Relicensed under the GNU GPL under clause 3 of the LGPL
Original copyright:
   Copyright  1999 Mitch Blevins <mblevin@debian.org>
   GNU LGPL License
   Based on LinkHistoryHack by Ben Darnel <bgdarnel@unity.ncsu.edu>
   which was...
   Based on SampleHack by Roger Chaplin <foursquaredev@att.net>
*/


#include <Common.h>
#include <System/SysAll.h>
#include <UI/UIAll.h>
#include "dial.h"
#include "dialRsc.h"

static void PrefsInit(FormPtr frm);
static void PrefsOK();
static VoidPtr GetObjectPtr(Word id);
static void SetList(Word triggerid, Word listid, UInt key);

// This is the code that will be called whenever the user taps myhack's "+"
// button in the HackMaster list. HackMaster is not smart about the content of
// this file; it will simply jump to the first executable statement in this
// file. If you need helper functions, prototype them above, and define them
// AFTER the trap patch.
Boolean PrefsHandler(EventPtr event)
{
   FormPtr         frm;
   Boolean handled=false;

   switch (event->eType) {
   case frmOpenEvent:
      frm = FrmGetActiveForm();
      PrefsInit(frm);
      FrmDrawForm(frm);
      handled=1;
      break;
      
   case ctlSelectEvent:
      switch (event->data.ctlEnter.controlID) {
      case PrefsOKButton:
	 PrefsOK();
	 /* FALL THRU */

      case PrefsCancelButton:
	 FrmGotoForm(9000);  // return to HackMaster control panel
	 handled=true;
	 break;
      }
      break;
      
   default:
      break;
   }
   
   return handled;
}

static void PrefsInit(FormPtr frm)
{
   MyPrefsType prefs;
   Word prefssize=sizeof(prefs);

   MemSet(&prefs, sizeof(MyPrefsType), 0);
   if ((PrefGetAppPreferences('Dhck', 0, &prefs, &prefssize, true)
	==noPreferenceFound)||(prefssize<sizeof(prefs))) {
      prefs.selectstart=calcChr;
      prefs.selectend=findChr;
      prefs.clipstart=findChr;
      prefs.clipend=calcChr;
      prefs.flags |= DBSC_FLAGS_JUMP;
   }

   SetList(SelectFromTrigger, SelectFromList, prefs.selectstart);
   SetList(SelectToTrigger, SelectToList, prefs.selectend);
   SetList(ClipFromTrigger, ClipFromList, prefs.clipstart);
   SetList(ClipToTrigger, ClipToList, prefs.clipend);
}

static void SetList(Word triggerid, Word listid, UInt key)
{
   Word selection;
   CharPtr text;
   
   switch (key) {
   case launchChr:
      selection=0;
      text="Applications";
      break;

   case menuChr:
      selection=1;
      text="Menu";
      break;

   case calcChr:
      selection=2;
      text="Calculator";
      break;

   case findChr:
      selection=3;
      text="Find";
      break;

   default:
      selection=4;
      text="Disabled";
      break;
   };
   LstSetSelection(GetObjectPtr(listid), selection);
   CtlSetLabel(GetObjectPtr(triggerid), text);
}

static void PrefsOK()
{
   UInt key[5];
   MyPrefsType prefs;
   Word prefssize=sizeof(prefs);

   key[0]=launchChr;
   key[1]=menuChr;
   key[2]=calcChr;
   key[3]=findChr;
   key[4]=0;
   
   prefs.selectstart=key[LstGetSelection(GetObjectPtr(SelectFromList))];
   prefs.selectend=key[LstGetSelection(GetObjectPtr(SelectToList))];
   prefs.clipstart=key[LstGetSelection(GetObjectPtr(ClipFromList))];
   prefs.clipend=key[LstGetSelection(GetObjectPtr(ClipToList))];

   PrefSetAppPreferences('Dhck', 0, 0, &prefs, prefssize, true);
}

static VoidPtr GetObjectPtr(Word id)
{
   FormPtr frm;
   frm=FrmGetActiveForm();
   return FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, id));
}
